function [SOM]=TrainSOM(SOM,OrderPhaseLearningRate,TunePhaseLearningRate,TunePhaseNeighborhoodDistance,Epochs,Data);

% Allena una SOM.

% Numero di esempi nel dataset
NumberOfData=length(Data(:,1));

% Numero di epoche (e di esempi) della Ordering phase e della tuning phase
NumberOfOrderPhaseEpochs=round(0.8*Epochs);
NumberOfOrderPhaseData=NumberOfOrderPhaseEpochs*NumberOfData;
NumberOfTunePhaseEpochs=Epochs-NumberOfOrderPhaseEpochs;
NumberOfTunePhaseData=NumberOfTunePhaseEpochs*NumberOfData;

% Neighborhood Distance iniziale
OrderPhaseNeighborhoodDistance=max(SOM.TopologicalDistancesLUT(:))/5;
           
% Waitbar
W=waitbar(0,'SOM training...');

% Addestramento
for Index=1:NumberOfOrderPhaseData
    
    % Learning rate
    LearningRate=(OrderPhaseLearningRate-(OrderPhaseLearningRate-TunePhaseLearningRate)*(Index-1)/(NumberOfOrderPhaseData-1));
    
    % Neighborhood distance
    NeighborhoodDistance=(OrderPhaseNeighborhoodDistance-(OrderPhaseNeighborhoodDistance-TunePhaseNeighborhoodDistance)*(Index-1)/(NumberOfOrderPhaseData-1));
    
    % Select casual input data
    InputPointIndex=round(1+(rand(1)*(NumberOfData-1)));
    InputPoint=Data(InputPointIndex,:);
    
    % Train
    SOM.Weights=SingleTrainSOM(SOM.Weights,SOM.NumberOfRows,SOM.NumberOfColumns,SOM.TopologicalPositions,SOM.TopologicalDistancesLUT,LearningRate,NeighborhoodDistance,InputPoint);

    % Waitbar
    if mod(Index,round(NumberOfData/20))==0
        waitbar(Index/NumberOfOrderPhaseData,W,['SOM training ' num2str(Index) '/' num2str(NumberOfOrderPhaseData) ' LR = ' num2str(mean(LearningRate)) ' ND = ' num2str(mean(NeighborhoodDistance))]);
        %PlotSOM(SOM);
        pause(0.01);
    end
    
end

% Waitbar
close (W);
